module Autosolver where

import Minesweeper

data MoveType = Uncover | Flag | None deriving (Eq, Show)

probabilityOfMine :: Board -> Square -> Float 
probabilityOfMine _ _ = 1.0

autoplay :: Board -> Board
autoplay b | fst (nextMove b) == Uncover = uncover b $ snd (nextMove b)
           | fst (nextMove b) == Flag = flag b $ snd (nextMove b)
           | otherwise = b

nextMove :: Board -> (MoveType, Square)
nextMove b | not . null $ uncoverStrat1 b = (Uncover, head $ uncoverStrat1 b)
           | not . null $ flagStrat1 b = (Flag, head $ flagStrat1 b)
           | otherwise = (None, (0,0))

-- filter:
-- uncovered squares
-- WITH at least one adjacent mine
-- WHERE the number of adjacent mines == number of adjacent flags
-- WHERE number of adjacent covered squares > number of adjacent flags
-- FIND adjacent squares
-- WHICH are valid
-- WHICH are covered
-- WHICH are not flagged
uncoverStrat1 :: Board -> [Square]
uncoverStrat1 b = 
    filter (not. isFlagged b) $
    filter (isCovered b) $
    filter (validSquare b) $
    concatMap adjacentSquares $
    filter (\s -> adjacentCovereds b s > adjacentFlags b s) $
    filter (\s -> adjacentFlags b s == adjacentMines b s) $
    filter (\s -> adjacentMines b s > 0) $ 
    uncoveredSquares b

-- filter:
-- uncovered squares
-- WITH at least one adjacent mine
-- WHERE the number of adjacent mines == number of adjacent covered squares
-- FIND adjacent squares
-- WHICH are valid
-- WHICH are covered
-- WHICH are not already flagged
flagStrat1 :: Board -> [Square]
flagStrat1 b =
    filter (not. isFlagged b) $
    filter (isCovered b) $
    filter (validSquare b) $
    concatMap adjacentSquares $
    filter (\s -> adjacentMines b s == adjacentCovereds b s) $
    filter (\s -> adjacentMines b s > 0) $ 
    uncoveredSquares b


-- if number of adjacent mines == number of adjacent flags, it's safe to uncover all adjacent non-flagged squares
adjacentMinesAccountedFor :: Board -> Square -> Bool 
adjacentMinesAccountedFor b (r,c) = adjacentMines b (r,c) == adjacentFlags b (r,c)